/*
 * 03_Interrupt.cpp
 *
 * Created: 2020
 *
 * Pindescription Arduino Uno
 * No.	Func.	PIN-MC	Used as In/Out Res. Pindefinition
 * 0	RX		PD0		RX		In
 * 1	TX		PD1		TX		In
 * 2	INT0	PD2		US-Echo	In
 * 3	Digit	PD3		Servo	Out
 * 4	Digit	PD4		Line-M	In
 * 5	OC0B	PD5		PWM-A	Out
 * 6	OC0B	PD6		PWM-B	Out
 * 7	PCINT23	PD7		Color-1	In	->	DDRD = 0b01101000
 * 8	Digit	PB0		Motor-A	Out
 * 9	Digit	PB1		Motor-B	Out
 *10	Digit	PB2		Line-L	In
 *11	PCINT3	PB3		Color-2	In
 *12	Digit	PB4		Infr.	In	Not needed
 *13	LED		PB5		LED		Out	->	DDRB = 0b00100011
 *A0	Ana		PC0		Line-RA	In
 *A1	Ana		PC1		Line-LA	In
 *A2	Ana		PC2		Dist-1	In
 *A3	Ana		PC3		Dist-2	In
 *A4	Digit	PC4		Line-R	In
 *A5	Digit	PC5		US-Trig	Out	->	DDRC = 0b00100000
 */ 
#define F_CPU 16000000UL //16MHz required for delay
#include <avr/io.h> //Input/Output library
#include <util/delay.h> //Needed for using _delay_...
#include <avr/interrupt.h> //External and internal Interrupts

//Definitions

//Macros
#define	LED_ON				PORTB |= (1 << PORTB5) //Push PORTB pin 5 high (LED, pin 13 "Uno")
#define	LED_OFF				PORTB &= ~(1 << PORTB5) //~ Not Operator
#define	LED_TOGGLE			PINB |= (1<<PINB5) //Toggle Arduino Uno pin 13


//Variables
unsigned char z; //For tests
unsigned int T2ck; //Counter TMR2 overflow

//Funktions, declaration
void Init (void); //Initialize Microcontroller


//Main routine
int main(void) {
	Init(); //Initalize the MC
	sei(); //Enable all interrupts
	//cli(); //Clear all interrupts
	while (1) {
		//Nothing to do!!!
	}
}

/***************************************************************************/
//Interrupt service routines (ISR)
//TMR2 overflow Interrupt
ISR(TIMER2_OVF_vect) { //Prescaler TMR2 1:8 = Interrupt every 128 us
//Just toggeling the LED
	T2ck++; //Counter TMR2 OVF
	if(T2ck == 2000) { //Approx. every 250ms (2000*128us = 256ms)
		T2ck = 0; //Dont forget!!
		LED_TOGGLE; //See above
	}
}

/***********************************************************************************************/
//Other functions

//Initialize the Microcontroller
void Init (void) {
	//Port settings (in brackets: pin of the arduino uno board, see above too)
	//0 = Input, 1 = Output
	DDRB = 0b00100011; //PortB Pin 5 to Output, LED on Arduino Uno board (13)
	//Timer2 Time divider
	TCCR2A = 0b00000000; //0000Normal port operation,00unimpl.,00WGM2[1:0]
	TCCR2B = 0b00000010; //0WGM22,010Prescaler 1:8
	TIMSK2 = 0b00000001; //Interrupt at TMR2-OVFL, 128us, 1s = 1000000us, 16MHz -> 16oscillations per us -> prescaler 1:8 -> 2 increments of TCNT2 per us -> overflow after 128us
}

/*Aufgabe: LED im 500ms-Tackt blinken lassen.
 *Dazu msst ihr nur ein wenig rechnen...*/


